@@ -9,41 +9,45 @@ import re
 import subprocess
 import locale
 import codecs
-from functools import wraps
 import unicodedata
+from functools import wraps
+from io import StringIO
+from urllib.parse import quote as urlquote
 
 import click
 
-from cloudify._compat import StringIO, urlquote
+from cloudify.models_states import AgentState
 from cloudify_rest_client.constants import VisibilityState
 from cloudify_rest_client.exceptions import NotModifiedError
 from cloudify_rest_client.exceptions import CloudifyClientError
 from cloudify_rest_client.exceptions import MaintenanceModeActiveError
 from cloudify_rest_client.exceptions import MaintenanceModeActivatingError
 
-from .. import env, logger
-from ..cli import helptexts
-from ..inputs import inputs_to_dict
-from ..utils import generate_random_string
-from ..constants import DEFAULT_BLUEPRINT_PATH
-from ..exceptions import SuppressedCloudifyCliError
-from ..exceptions import (LabelsValidationError,
-                          CloudifyBootstrapError,
-                          CloudifyValidationError,)
-from ..logger import (
+from cloudify_cli import env, logger
+from cloudify_cli.cli import helptexts
+from cloudify_cli.constants import DEFAULT_BLUEPRINT_PATH
+from cloudify_cli.exceptions import (
+    LabelsValidationError,
+    CloudifyBootstrapError,
+    CloudifyValidationError,
+    SuppressedCloudifyCliError)
+from cloudify_cli.filters_utils import (
+    get_filter_rules,
+    create_labels_filter_rules_list,
+    create_attributes_filter_rules_list)
+from cloudify_cli.inputs import inputs_to_dict
+from cloudify_cli.logger import (
     get_logger,
     set_global_verbosity_level,
     DEFAULT_LOG_FILE,
     set_global_json_output,
     set_global_extended_view)
-from ..filters_utils import (get_filter_rules,
-                             create_labels_filter_rules_list,
-                             create_attributes_filter_rules_list)
+from cloudify_cli.utils import generate_random_string
 
 
 CLICK_CONTEXT_SETTINGS = dict(
     help_option_names=['-h', '--help'],
-    token_normalize_func=lambda param: param.lower())
+)
 
 AGENT_FILTER_NODE_IDS = 'node_ids'
 AGENT_FILTER_NODE_INSTANCE_IDS = 'node_instance_ids'
@@ -164,21 +168,26 @@ def show_version(ctx, param, value):
     if not value or ctx.resilient_parsing:
         return
 
-    cli_version_data = {
-        'version': pkg_resources.require('cloudify')[0].version
-    }
-    rest_version_data = env.get_manager_version_data() \
-        if env.is_manager_active() else None
+    cli_version_output = _format_version_data(
+        {'version': pkg_resources.require('cloudify')[0].version},
+        prefix='Cloudify CLI ',
+        infix=' ' * 5,
+        suffix='\n')
+
+    try:
+        rest_version_data = env.get_manager_version_data() \
+            if env.is_manager_active() else None
+    except Exception as e:
+        get_logger().info(cli_version_output)
+        sys.stderr.write("Cannot get Cloudify Manager version. {}: "
+                         "{}\n".format(type(e).__name__, str(e)))
+        ctx.exit(1)
+
     output = ''
     if rest_version_data:
         edition = rest_version_data['edition'].title()
         output += '{0} edition\n\n'.format(edition)
-
-    output += _format_version_data(
-        cli_version_data,
-        prefix='Cloudify CLI ',
-        infix=' ' * 5,
-        suffix='\n')
+    output += cli_version_output
     if rest_version_data:
         output += _format_version_data(
             rest_version_data,
@@ -439,7 +448,7 @@ def set_cli_except_hook(global_verbosity_level):
     def new_excepthook(tpe, value, tb):
         with open(DEFAULT_LOG_FILE, 'a') as log_file:
             traceback.print_exception(
-                etype=tpe,
+                tpe,
                 value=value,
                 tb=tb,
                 file=log_file)
@@ -468,7 +477,7 @@ def set_cli_except_hook(global_verbosity_level):
             # print traceback if verbose
             s_traceback = StringIO()
             traceback.print_exception(
-                etype=tpe,
+                tpe,
                 value=value,
                 tb=tb,
                 file=s_traceback)
@@ -887,6 +896,21 @@ class Options(object):
             is_flag=True,
             help=helptexts.SKIP_REINSTALL)
 
+        self.skip_drift_check = click.option(
+            '--skip-drift-check',
+            is_flag=True,
+            help=helptexts.SKIP_DRIFT_CHECK)
+
+        self.force_reinstall = click.option(
+            '--force-reinstall',
+            is_flag=True,
+            help=helptexts.FORCE_REINSTALL)
+
+        self.skip_heal = click.option(
+            '--skip-heal',
+            is_flag=True,
+            help=helptexts.SKIP_HEAL)
+
         self.dont_skip_reinstall = click.option(
             '--dont-skip-reinstall',
             is_flag=True,
@@ -965,6 +989,13 @@ class Options(object):
             help=helptexts.PROFILE_MANAGER_IP,
         )
 
+        self.manager_token = click.option(
+            '-T',
+            '--manager-token',
+            required=False,
+            help=helptexts.MANAGER_TOKEN,
+        )
+
         self.manager_username = click.option(
             '-u',
             '--manager-username',
@@ -1253,6 +1284,33 @@ class Options(object):
             required=False,
             help=helptexts.SECRET_STRING)
 
+        self.secret_schema = click.option(
+            '--schema',
+            'secret_schema',
+            required=False,
+            default=None,
+            cls=MutuallyExclusiveOption,
+            mutually_exclusive=['dict', 'list'],
+            help=helptexts.SECRET_SCHEMA)
+
+        self.secret_flag_dict = click.option(
+            '--dict',
+            'secret_flag_dict',
+            is_flag=True,
+            default=False,
+            cls=MutuallyExclusiveOption,
+            mutually_exclusive=['schema', 'list'],
+            help=helptexts.SECRET_FLAG_DICT)
+
+        self.secret_flag_list = click.option(
+            '--list',
+            'secret_flag_list',
+            is_flag=True,
+            default=False,
+            cls=MutuallyExclusiveOption,
+            mutually_exclusive=['schema', 'dict'],
+            help=helptexts.SECRET_FLAG_LIST)
+
         self.secret_update_if_exists = click.option(
             '-u',
             '--update-if-exists',
@@ -1368,6 +1426,33 @@ class Options(object):
             help=helptexts.QUEUE_SNAPSHOTS
         )
 
+        self.tempdir_path = click.option(
+            '--tempdir-path',
+            help=helptexts.TEMPDIR_PATH
+        )
+
+        self.legacy = click.option(
+            '--legacy/--no-legacy',
+            is_flag=True,
+            default=True,
+            help=helptexts.LEGACY_SNAPSHOT
+        )
+
+        self.wait_for_status = click.option(
+            '-w',
+            '--wait-for-status',
+            is_flag=True,
+            default=False,
+            help=helptexts.WAIT_FOR_STATUS
+        )
+
+        self.queue_log_bundle = click.option(
+            '--queue',
+            is_flag=True,
+            default=False,
+            help=helptexts.QUEUE_LOG_BUNDLES,
+        )
+
         self.wait_after_fail = click.option(
             '--wait-after-fail',
             default=600,
@@ -1789,14 +1874,69 @@ class Options(object):
             help=helptexts.DEPENDENCIES_OF
         )
 
+        self.execution_group_concurrency = click.option(
+            '--concurrency',
+            type=int,
+            default=5,
+            help=helptexts.EXECUTION_GROUP_CONCURRENCY
+        )
+
+        self.worker_names = click.option(
+            '-w', '--with-worker-names/--without-worker-names',
+            'with_worker_names',
+            is_flag=True,
+            help=helptexts.WORKER_NAMES,
+            default=False,
+        )
+
+        self.drift_only = click.option(
+            '--drift-only',
+            is_flag=True,
+            help=helptexts.DRIFT_ONLY,
+            default=False,
+            cls=MutuallyExclusiveOption,
+            mutually_exclusive=['blueprint_id', 'blueprint_path', 'inputs'],
+        )
+
+        self.secrets_provider_name = click.option(
+            '--name',
+            'secrets_provider_name',
+            required=True,
+            callback=validate_value_not_empty,
+            help=helptexts.SECRETS_PROVIDER_NAME,
+        )
+
+        self.provider_name = click.option(
+            '-p',
+            '--provider',
+            'provider_name',
+            required=False,
+            callback=validate_value_not_empty,
+            help=helptexts.SECRETS_PROVIDER_NAME,
+        )
+
+        self.evaluate_functions = click.option(
+            '--evaluate-functions',
+            is_flag=True,
+            default=False,
+            required=False,
+            help=helptexts.EVALUATE_FUNCTIONS,
+        )
+        self.recursive_delete = click.option(
+            '--recursive',
+            default=False,
+            is_flag=True,
+            help=helptexts.RECURSIVE_DELETE,
+        )
+
     def common_options(self, f):
         """A shorthand for applying commonly used arguments.
 
         To be used for arguments that are going to be applied for all or
         almost all commands.
         """
-        for arg in [self.json, self.verbose(), self.format, self.quiet(),
-                    self.manager]:
+        for arg in [self.manager, self.json, self.format,
+                    self.verbose(), self.quiet()]:
             f = arg(f)
         return f
 
@@ -1824,22 +1964,41 @@ class Options(object):
 
         Applies deployment id, node id and node instance id filters.
         """
-        node_instance_id = click.option('--node-instance-id', multiple=True,
-                                        help=helptexts.AGENT_NODE_INSTANCE_ID,
-                                        callback=self.parse_comma_separated)
-        node_id = click.option('--node-id', multiple=True,
-                               help=helptexts.AGENT_NODE_ID,
-                               callback=self.parse_comma_separated)
-        install_method = click.option('--install-method', multiple=True,
-                                      help=helptexts.AGENT_INSTALL_METHOD,
-                                      callback=self.parse_comma_separated)
-        deployment_id = click.option('--deployment-id', multiple=True,
-                                     help=helptexts.AGENT_DEPLOYMENT_ID,
-                                     callback=self.parse_comma_separated)
+        node_instance_id = click.option(
+            '--node-instance-id',
+            multiple=True,
+            help=helptexts.AGENT_NODE_INSTANCE_ID,
+            callback=self.parse_comma_separated,
+        )
+        node_id = click.option(
+            '--node-id',
+            multiple=True,
+            help=helptexts.AGENT_NODE_ID,
+            callback=self.parse_comma_separated,
+        )
+        install_method = click.option(
+            '--install-method',
+            multiple=True,
+            help=helptexts.AGENT_INSTALL_METHOD,
+            callback=self.parse_comma_separated,
+        )
+        deployment_id = click.option(
+            '--deployment-id',
+            multiple=True,
+            help=helptexts.AGENT_DEPLOYMENT_ID,
+            callback=self.parse_comma_separated,
+        )
+        all_states = click.option(
+            '--all-states',
+            default=False,
+            is_flag=True,
+            help=helptexts.AGENT_ALL_STATES,
+        )
 
         # we add separate --node-instance-id, --node-id and --deployment-id
         # arguments, but only expose a agents_filter = {'node_id': ..} dict
         # to the decorated function
+
         def _filters_deco(f):
             @wraps(f)
             def _inner(*args, **kwargs):
@@ -1851,12 +2010,14 @@ class Options(object):
                         ('install_method', AGENT_FILTER_INSTALL_METHODS)]:
                     filters[filter_name] = kwargs.pop(arg_name, None)
 
+                if not kwargs.pop('all_states', False):
+                    filters['state'] = [AgentState.STARTED]
                 kwargs['agent_filters'] = filters
                 return f(*args, **kwargs)
             return _inner
 
         for arg in [install_method, node_instance_id, node_id,
-                    deployment_id, _filters_deco]:
+                    deployment_id, all_states, _filters_deco]:
             f = arg(f)
         return f
 
@@ -2042,12 +2203,16 @@ class Options(object):
             help=helptexts.EXECUTION_ID)
 
     @staticmethod
-    def blueprint_id(required=False, validate=False):
+    def blueprint_id(
+        required=False,
+        validate=False,
+        help=helptexts.BLUEPRINT_ID,
+    ):
         return click.option(
             '-b',
             '--blueprint-id',
             required=required,
-            help=helptexts.BLUEPRINT_ID,
+            help=help,
             callback=_get_validate_callback(validate))
 
     @staticmethod
@@ -2105,6 +2270,7 @@ class Options(object):
             '-y',
             '--yaml-path',
             required=True,
+            multiple=True,
             help=helptexts.PLUGIN_YAML_PATH)
 
     @staticmethod
@@ -2351,5 +2517,100 @@ class Options(object):
     def deployment_filter_rules(self, f):
         return self._filter_rules(f, 'deployment')
 
+    @staticmethod
+    def secrets_provider_type(
+            required=True,
+            _help=None,
+            default=None,
+            callback=validate_value_not_empty,
+    ):
+        args = [
+            '-y',
+            '--type',
+            'secrets_provider_type',
+        ]
+        kwargs = {
+            'required': required,
+            'help': _help or helptexts.SECRETS_PROVIDER_TYPE,
+            'callback': callback,
+        }
+
+        if default is not None:
+            kwargs['default'] = default
+
+        return click.option(*args, **kwargs)
+
+    @staticmethod
+    def secrets_provider_skip_check():
+        return click.option(
+            '-s',
+            '--skip-check',
+            is_flag=True,
+            default=False,
+            required=False,
+            help=helptexts.SECRETS_PROVIDER_SKIP_CHECK,
+        )
+
+    @staticmethod
+    def connection_parameters(required=True, _help=None, default=None):
+        args = [
+            '-c',
+            '--connection-parameters',
+        ]
+        kwargs = {
+            'required': required,
+            'help': _help or helptexts.SECRETS_PROVIDER_CONNECTION_PARAMETERS,
+            'callback': inputs_callback,
+            'multiple': True,
+            'default': default,
+        }
+
+        return click.option(*args, **kwargs)
+
+    @staticmethod
+    def provider_options(required=True, _help=None, default=None):
+        args = [
+            '-o',
+            '--provider-options',
+        ]
+        kwargs = {
+            'required': required,
+            'help': _help or helptexts.SECRETS_PROVIDER_OPTIONS,
+            'callback': inputs_callback,
+            'multiple': True,
+            'default': default,
+        }
+
+        return click.option(*args, **kwargs)
+
+    @staticmethod
+    def provider_multiple(required=False, _help=None, default=None):
+        args = [
+            '-p',
+            '--provider',
+            'provider',
+        ]
+        kwargs = {
+            'required': required,
+            'help': _help or helptexts.SECRETS_PROVIDER_NAME_MULTIPLE,
+            'multiple': True,
+            'default': default,
+        }
+
+        return click.option(*args, **kwargs)
+
 
 options = Options()
+
+
+class SummaryArgs(click.Choice):
+    """
+    Used for correctly displaying usage of summary commands (e.g. `cfy
+    blueprints summary`) in which the user must choose a field to summarize
+    by from a list.
+    We want  Usage: cfy blueprints summary [OPTIONS] TARGET_FIELD [SUB_FIELD]
+    Not      Usage: cfy blueprints summary [OPTIONS]
+                    [visibility|tenant_name] [[visibility|tenant_name]]
+    """
+    def get_metavar(self, param):
+        pass
