@@ -91,6 +91,9 @@ SKIP_REINSTALL = (
     "been modified, as part of a deployment update. Node instances that were "
     "explicitly given to the reinstall list will still be reinstalled"
 )
+SKIP_DRIFT_CHECK = "Skip running check_drift during deployment update"
+FORCE_REINSTALL = "Reinstall all changed nodes, don't run update operations"
+SKIP_HEAL = "Skip running heal and check_status before the update"
 DONT_SKIP_REINSTALL = (
     "Reinstall node-instances that their properties have been modified as part"
     " of a deployment update. Node instances that were explicitly specified"
@@ -198,6 +201,7 @@ PROFILE_MANAGER_IP = 'The address of the Manager'
 SSH_USER = 'The SSH user on the manager host machine'
 SSH_KEY = 'The path to the ssh key-file to use when connecting'
 SSH_PORT = 'The SSH port to use when connecting to the manager'
+MANAGER_TOKEN = 'Manager token used to run commands on the manager'
 MANAGER_USERNAME = 'Manager username used to run commands on the manager'
 MANAGER_PASSWORD = 'Manager password used to run commands on the manager'
 MANAGER_TENANT = 'The tenant associated with the current user operating the ' \
@@ -331,6 +335,10 @@ GET_DATA = 'When set to True, displays the full list of connected resources ' \
 PROFILE_NAME = 'Name of the profile to use'
 SECRET_VALUE = "The secret's value to be set"
 SECRET_STRING = "The string to use as the secret's value"
+SECRET_FLAG_DICT = "Whether the secret is to be treated as a dict"
+SECRET_FLAG_LIST = "Whether the secret is to be treated as a lists"
+SECRET_SCHEMA = "A JSON schema against which the secret will be validated [" \
+                "default: '{\"type\": \"string\"}']"
 SECRET_FILE = "The file with the contents of the secret"
 SECRET_UPDATE_IF_EXISTS = 'Update secret value if secret key already ' \
                           'exists. [This option is deprecated; use cfy ' \
@@ -381,6 +389,9 @@ IGNORE_PLUGIN_FAILURE = 'If set, plugin installation errors during snapshot ' \
                         'not fail the snapshot restore workflow'
 QUEUE_SNAPSHOTS = 'If set, snapshot-creation-workflows that can`t currently ' \
                   'run will be queued and run automatically when possible'
+QUEUE_LOG_BUNDLES = 'If set, log-bundle-creation-workflows that can`t ' \
+                    'currently run will be queued and run automatically ' \
+                    'when possible'
 QUEUE_EXECUTIONS = 'If set, executions that can`t currently run will be '\
                    'queued and run automatically when possible'
 
@@ -422,6 +433,7 @@ AGENT_NODE_ID = 'The node id to be used for filtering' \
 AGENT_INSTALL_METHOD = 'Only show agents installed with this install_method' \
                        + _MULTIPLE_TIMES_FRAGMENT
 AGENT_DEPLOYMENT_ID = DEPLOYMENT_ID + _MULTIPLE_TIMES_FRAGMENT
+AGENT_ALL_STATES = 'Show agents in all states, not only started ones'
 
 AGENTS_WAIT = "Wait for agents operations to end, and show execution logs"
 INSTALL_AGENT_TIMEOUT = "Agent installation timeout"
@@ -549,6 +561,7 @@ DEP_GROUP_INTO_ENVIRONMENTS = 'Add created deployments to the environments ' \
                               'already existing in this group.'
 GROUP_ID_FILTER = 'Show only results belonging to this group'
 DELETE_GROUP_DEPLOYMENTS = 'Delete all deployments belonging to this group'
+EXECUTION_GROUP_CONCURRENCY = 'Run this many executions at a time'
 
 GENERATE_ID = 'Generate a UUID to serve as the deployment ID. This flag ' \
               'cannot be provided if a deployment ID is specified'
@@ -582,3 +595,33 @@ AUDIT_TRUNCATE_BEFORE = 'Truncate audit logs which were stored this long ' \
 
 SET_USERNAME = 'The name of the user who will be the new owner '\
                'of the resource.'
+WORKER_NAMES = 'Show the worker name for each event'
+DRIFT_ONLY = 'Run update without changing anything. This will still check ' \
+             'drift and run update operations as necessary'
+TEMPDIR_PATH = "Temporary location to be used for snapshot creation. If not " \
+               "specified, /tmp will be used."
+LEGACY_SNAPSHOT = "Create legacy version of the snapshot (as opposed to 'new')"
+WAIT_FOR_STATUS = "Whether to wait for snapshot status [default: False]."
+SUMMARY_HELP = """
+    Retrieve summary of {type}, e.g. a count of each {example}.
+
+    `TARGET_FIELD` is the field to summarize {type} on. `SUB_FIELD` is an
+    optional second field to summarize {type} on. Both can be chosen from
+    [{fields}].
+
+    E.g. `cfy {type} summary tenant_name visibility` will summarize
+    {type} by tenant_name with a secondary grouping by visibility.
+    """
+SECRETS_PROVIDER_NAME = "Secrets Provider's name"
+SECRETS_PROVIDER_NAME_MULTIPLE = "Secrets Provider's name list"
+SECRETS_PROVIDER_SKIP_CHECK = "Do not check connectivity to secrets provider."
+SECRETS_PROVIDER_TYPE = "Secrets Provider's type"
+SECRETS_PROVIDER_CONNECTION_PARAMETERS = """
+    Secrets Provider's connection parameters in stringify JSON format
+    """
+SECRETS_PROVIDER_OPTIONS = """
+    Secrets Provider's options in stringify JSON format
+    """
+EVALUATE_FUNCTIONS = "Evaluate functions in returned nodes and node instances"
+RECURSIVE_DELETE = 'Recursively delete all service deployments contained in ' \
+                   'this deployment'
