@@ -17,31 +17,38 @@
 import os
 import json
 import shutil
+from urllib.parse import urlparse
 
 import click
 
 from dsl_parser.parser import parse_from_path
 from dsl_parser.exceptions import DSLParsingException
-from cloudify._compat import urlparse
 from cloudify_rest_client.constants import VISIBILITY_EXCEPT_PRIVATE
 
-from .. import env
-from .. import local
-from .. import utils
-from ..cli import cfy, helptexts
-from .. import blueprint
-from .. import exceptions
-from ..config import config
-from ..logger import get_global_json_output
-from ..table import print_data, print_single
-from ..exceptions import CloudifyCliError
-from ..utils import prettify_client_error, get_visibility, validate_visibility
-from ..labels_utils import (add_labels,
-                            delete_labels,
-                            list_labels,
-                            serialize_resource_labels)
-from .. import filters_utils
-from .summary import BASE_SUMMARY_FIELDS, structure_summary_results
+from cloudify_cli import (
+    blueprint,
+    env,
+    exceptions,
+    filters_utils,
+    local,
+    utils)
+from cloudify_cli.cli import cfy, helptexts
+from cloudify_cli.config import config
+from cloudify_cli.exceptions import CloudifyCliError
+from cloudify_cli.labels_utils import (
+    add_labels,
+    delete_labels,
+    list_labels,
+    serialize_resource_labels)
+from cloudify_cli.logger import get_global_json_output
+from cloudify_cli.table import print_data, print_single
+from cloudify_cli.utils import (
+    prettify_client_error,
+    get_visibility,
+    validate_visibility)
+from cloudify_cli.commands.summary import (
+    BASE_SUMMARY_FIELDS,
+    structure_summary_results)
 
 
 DESCRIPTION_LIMIT = 20
@@ -97,7 +104,8 @@ def upload(ctx,
     retrieved from GitHub).
     Supported archive types are: zip, tar, tar.gz and tar.bz2
     """
-    client.license.check()
+    if client.manager.get_version().get('edition') == 'premium':
+        client.license.check()
     utils.explicit_tenant_name_message(tenant_name, logger)
     processed_blueprint_path = blueprint.get(
         blueprint_path, blueprint_filename, icon_path)
@@ -461,8 +469,8 @@ def set_global(blueprint_id, logger, client):
     with prettify_client_error(status_codes, logger):
         client.blueprints.set_global(blueprint_id)
         logger.info('Blueprint `{0}` was set to global'.format(blueprint_id))
-        logger.info("This command will be deprecated soon, please use the "
-                    "'set-visibility' command instead")
+        logger.warning("This command is deprecated and will be removed soon, "
+                       "please use the 'set-visibility' command instead")
 
 
 @blueprints.command(name='set-visibility',
@@ -488,9 +496,13 @@ def set_visibility(blueprint_id, visibility, logger, client):
 
 @blueprints.command(name='summary',
                     short_help='Retrieve summary of blueprint details '
-                               '[manager only]')
-@cfy.argument('target_field', type=click.Choice(BLUEPRINTS_SUMMARY_FIELDS))
-@cfy.argument('sub_field', type=click.Choice(BLUEPRINTS_SUMMARY_FIELDS),
+                               '[manager only]',
+                    help=helptexts.SUMMARY_HELP.format(
+                        type='blueprints',
+                        example='blueprint with the same tenant name',
+                        fields='|'.join(BLUEPRINTS_SUMMARY_FIELDS)))
+@cfy.argument('target_field', type=cfy.SummaryArgs(BLUEPRINTS_SUMMARY_FIELDS))
+@cfy.argument('sub_field', type=cfy.SummaryArgs(BLUEPRINTS_SUMMARY_FIELDS),
               default=None, required=False)
 @cfy.options.common_options
 @cfy.options.tenant_name(required=False, resource_name_for_help='summary')
@@ -499,11 +511,6 @@ def set_visibility(blueprint_id, visibility, logger, client):
 @cfy.pass_client()
 def summary(target_field, sub_field, logger, client, tenant_name,
             all_tenants):
-    """Retrieve summary of blueprints, e.g. a count of each blueprint with
-    the same tenant name.
-
-    `TARGET_FIELD` is the field to summarise blueprints on.
-    """
     utils.explicit_tenant_name_message(tenant_name, logger)
     logger.info('Retrieving summary of blueprints on field {field}'.format(
         field=target_field))
