@@ -20,23 +20,29 @@ import time
 import click
 from cloudify_rest_client import exceptions
 
-from .. import local
-from .. import utils
-from ..table import print_data, print_single, print_details
-from ..utils import get_deployment_environment_execution
-from ..cli import cfy, helptexts
-from ..logger import (get_events_logger,
-                      get_global_json_output,
-                      get_global_extended_view)
-from ..constants import DEFAULT_UNINSTALL_WORKFLOW, CREATE_DEPLOYMENT
-from ..execution_events_fetcher import (
+from cloudify_cli import local, utils
+from cloudify_cli.cli import cfy, helptexts
+from cloudify_cli.constants import (
+    DEFAULT_UNINSTALL_WORKFLOW,
+    CREATE_DEPLOYMENT)
+from cloudify_cli.exceptions import (
+    CloudifyCliError,
+    ExecutionTimeoutError,
+    SuppressedCloudifyCliError)
+from cloudify_cli.execution_events_fetcher import (
     ExecutionEventsFetcher,
     wait_for_execution,
-    wait_for_execution_group
-)
-from ..exceptions import CloudifyCliError, ExecutionTimeoutError, \
-    SuppressedCloudifyCliError
-from .summary import BASE_SUMMARY_FIELDS, structure_summary_results
+    wait_for_execution_group)
+from cloudify_cli.logger import (
+    get_events_logger,
+    get_global_json_output,
+    get_global_extended_view)
+from cloudify_cli.table import print_data, print_single, print_details
+from cloudify_cli.utils import get_deployment_environment_execution
+
+from cloudify_cli.commands.summary import (
+    BASE_SUMMARY_FIELDS,
+    structure_summary_results)
 
 _STATUS_CANCELING_MESSAGE = (
     'NOTE: Executions currently in a "canceling/force-canceling" status '
@@ -180,6 +186,7 @@ def manager_list(
 @cfy.options.json_output
 @cfy.options.dry_run
 @cfy.options.wait_after_fail
+@cfy.options.worker_names
 @cfy.options.common_options
 @cfy.options.tenant_name(required=False, resource_name_for_help='execution')
 @cfy.options.schedule
@@ -199,6 +206,7 @@ def manager_start(workflow_id,
                   wait_after_fail,
                   queue,
                   schedule,
+                  with_worker_names,
                   logger,
                   client,
                   tenant_name):
@@ -207,7 +215,7 @@ def manager_start(workflow_id,
     `WORKFLOW_ID` is the id of the workflow to execute (e.g. `uninstall`)
     """
     utils.explicit_tenant_name_message(tenant_name, logger)
-    events_logger = get_events_logger(json_output)
+    events_logger = get_events_logger(json_output, with_worker_names)
     events_message = "* Run 'cfy events list {0}' to retrieve the " \
                      "execution's events/logs"
     original_timeout = timeout
@@ -385,9 +393,13 @@ def manager_resume(execution_id, reset_operations, logger, client,
 
 @executions.command(name='summary',
                     short_help='Retrieve summary of execution details '
-                               '[manager only]')
-@cfy.argument('target_field', type=click.Choice(EXECUTIONS_SUMMARY_FIELDS))
-@cfy.argument('sub_field', type=click.Choice(EXECUTIONS_SUMMARY_FIELDS),
+                               '[manager only]',
+                    help=helptexts.SUMMARY_HELP.format(
+                        type='executions',
+                        example='execution with the same deployment ID',
+                        fields='|'.join(EXECUTIONS_SUMMARY_FIELDS)))
+@cfy.argument('target_field', type=cfy.SummaryArgs(EXECUTIONS_SUMMARY_FIELDS))
+@cfy.argument('sub_field', type=cfy.SummaryArgs(EXECUTIONS_SUMMARY_FIELDS),
               default=None, required=False)
 @cfy.options.common_options
 @cfy.options.tenant_name(required=False, resource_name_for_help='summary')
@@ -397,11 +409,6 @@ def manager_resume(execution_id, reset_operations, logger, client,
 @cfy.pass_client()
 def summary(target_field, sub_field, group_id, logger, client, tenant_name,
             all_tenants):
-    """Retrieve summary of executions, e.g. a count of each execution with
-    the same deployment ID.
-
-    `TARGET_FIELD` is the field to summarise executions on.
-    """
     utils.explicit_tenant_name_message(tenant_name, logger)
     logger.info('Retrieving summary of executions on field {field}'.format(
         field=target_field))
@@ -469,10 +476,73 @@ def delete(logger, client, tenant_name, to_datetime, before, keep_last,
         logger.info('\nNo executions to delete')
 
 
+@executions.group('graphs')
+@cfy.options.common_options
+def graphs():
+    """Handle executions' tasks-graphs"""
+
+
+@graphs.command('list')
+@cfy.options.common_options
+@cfy.argument('execution-id', required=True)
+@click.option('--name', help='List graphs with this name')
+@cfy.pass_logger
+@cfy.pass_client()
+def graphs_list(execution_id, name, client, logger):
+    """List tasks-graphs for an execution"""
+    tgs = client.tasks_graphs.list(execution_id=execution_id, name=name)
+    print_data(['id', 'name', 'created_at'], tgs,
+               'Tasks graphs of execution {0}:'.format(execution_id))
+
+
+@executions.group('operations')
+@cfy.options.common_options
+def operations():
+    """Handle executions' operations"""
+
+
+@operations.command('get')
+@cfy.options.common_options
+@cfy.argument('operation-id', required=True)
+@cfy.pass_logger
+@cfy.pass_client()
+def operations_get(operation_id, client, logger):
+    """Display the details of an operation"""
+    op = client.operations.get(operation_id)
+    print_details(op, 'Operation {0}:'.format(operation_id))
+
+
+@operations.command('list')
+@cfy.options.common_options
+@cfy.argument('execution-id', required=False)
+@click.option('--graph-id', required=False,
+              help='List operations of this graph '
+                   '(exclusive with execution-id)')
+@click.option('--show-internal', type=bool, is_flag=True, default=False,
+              help='Also list internal operations')
+@click.option('--state', required=False, help='List operations in this state')
+@cfy.pass_logger
+@cfy.pass_client()
+def operations_list(execution_id, graph_id, state, show_internal,
+                    client, logger):
+    """List operations for an execution or a graph"""
+    ops = client.operations.list(
+        graph_id=graph_id,
+        execution_id=execution_id,
+        state=state,
+        skip_internal=not show_internal,
+    )
+    print_data(
+        ['id', 'name', 'type', 'state', 'manager_name', 'agent_name'],
+        ops,
+        'Operations'
+    )
+
+
 @executions.group('groups')
 @cfy.options.common_options
 def groups():
-    pass
+    """Manage execution groups"""
 
 
 @groups.command('get',
@@ -483,6 +553,11 @@ def groups():
 @cfy.pass_logger
 @cfy.options.extended_view
 def execution_groups_get(execution_group_id, client, logger):
+    """Display execution group information
+
+    This includes the source deployment group, and the workflow name.
+    """
+
     group = client.execution_groups.get(execution_group_id)
     print_single(
         ['id', 'workflow_id', 'deployment_group_id'],
@@ -506,6 +581,7 @@ def _format_group(g):
 @cfy.pass_logger
 @cfy.options.extended_view
 def execution_groups_list(client, logger):
+    """List all execution groups"""
     groups = [_format_group(g) for g in client.execution_groups.list()]
     print_data(
         ['id', 'workflow_id', 'deployment_group'],
@@ -522,6 +598,7 @@ def execution_groups_list(client, logger):
 @cfy.pass_logger
 @cfy.options.extended_view
 def execution_groups_details(execution_group_id, client, logger):
+    """Show execution group details"""
     group = client.execution_groups.get(execution_group_id)
     group.update({'#executions': len(group.get('execution_ids'))})
     print_single(['workflow_id', 'deployment_group_id', '#executions',
@@ -560,11 +637,11 @@ def execution_groups_details(execution_group_id, client, logger):
                 short_help='Execute a workflow on each deployment in a group')
 @click.option('--deployment-group', '-g',
               help='The deployment group ID to run the workflow on')
-@click.option('--concurrency', help='Run this many executions at a time',
-              type=int, default=5)
+@cfy.options.execution_group_concurrency
 @click.argument('workflow-id')
 @cfy.options.common_options
 @cfy.options.parameters
+@cfy.options.worker_names
 @cfy.options.json_output
 @cfy.options.force(help=helptexts.FORCE_CONCURRENT_EXECUTION)
 @cfy.options.timeout()
@@ -572,8 +649,13 @@ def execution_groups_details(execution_group_id, client, logger):
 @cfy.pass_logger
 def execution_groups_start(deployment_group, workflow_id, parameters,
                            json_output, force, timeout, concurrency,
-                           client, logger):
-    events_logger = get_events_logger(json_output)
+                           with_worker_names, client, logger):
+    """Start an execution group
+
+    This starts an execution on every deployment in the given deployment
+    group.
+    """
+    events_logger = get_events_logger(json_output, with_worker_names)
     group = client.execution_groups.start(
         deployment_group_id=deployment_group,
         workflow_id=workflow_id,
@@ -623,6 +705,11 @@ def execution_groups_start(deployment_group, workflow_id, parameters,
 @cfy.pass_logger
 def execution_groups_cancel(group_id, force, kill,
                             client, logger, tenant_name):
+    """Cancel an execution group
+
+    This cancels all running executions in the group. Executions that already
+    finished are unaffected.
+    """
     utils.explicit_tenant_name_message(tenant_name, logger)
     if kill:
         message = 'Killing'
@@ -645,6 +732,11 @@ def execution_groups_cancel(group_id, force, kill,
 @cfy.pass_logger
 def execution_groups_resume(group_id, reset_operations,
                             client, logger, tenant_name):
+    """Resume an execution group
+
+    This resumes all failed executions in the group. Executions that already
+    finished are unaffected.
+    """
     utils.explicit_tenant_name_message(tenant_name, logger)
     logger.info('Resuming execution group %s', group_id)
     client.execution_groups.resume(group_id, force=reset_operations)
@@ -685,18 +777,41 @@ def execution_groups_set_success(group_id, success_group_id,
 @cfy.pass_logger
 def execution_groups_set_failure(group_id, failure_group_id,
                                  client, logger, tenant_name):
-    """Set success target group for this execution-group.
+    """Set failure target group for this execution-group.
 
-    Deployments for which the execution succeeds, will be added to the
+    Deployments for which the execution fails, will be added to the
     success target deployments group.
     """
     utils.explicit_tenant_name_message(tenant_name, logger)
     client.execution_groups.set_target_group(
         group_id, failed_group=failure_group_id)
-    logger.info('Execution group %s: success target group set to %s',
+    logger.info('Execution group %s: failure target group set to %s',
                 group_id, failure_group_id)
 
 
+@groups.command('set-concurrency',
+                short_help='Change the concurrency for a group')
+@cfy.argument('group-id')
+@cfy.argument('concurrency', type=int)
+@cfy.options.tenant_name(
+    required=False, resource_name_for_help='execution group')
+@cfy.options.common_options
+@cfy.pass_client()
+@cfy.pass_logger
+def execution_groups_set_concurrency(
+    group_id, concurrency, client, logger, tenant_name,
+):
+    """Change the concurrency setting of an execution group.
+
+    When starting executions belonging to this group, the new concurrency
+    setting will be used. Already-running executions are unaffected.
+    """
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    client.execution_groups.set_concurrency(group_id, concurrency)
+    logger.info('Execution group %s: concurrency set to %s',
+                group_id, concurrency)
+
+
 @cfy.group(name='executions')
 @cfy.options.common_options
 def local_executions():
