@@ -1,7 +1,7 @@
 import click
 
-from ..cli import cfy
-from ..table import print_data
+from cloudify_cli.cli import cfy
+from cloudify_cli.table import print_data
 
 
 @cfy.group(name='permissions')
