@@ -1,39 +1,40 @@
-########
-# Copyright (c) 2014 GigaSpaces Technologies Ltd. All rights reserved
-#
-# Licensed under the Apache License, Version 2.0 (the "License");
-# you may not use this file except in compliance with the License.
-# You may obtain a copy of the License at
-#
-#        http://www.apache.org/licenses/LICENSE-2.0
-#
-# Unless required by applicable law or agreed to in writing, software
-# distributed under the License is distributed on an "AS IS" BASIS,
-# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
-# See the License for the specific language governing permissions and
-# limitations under the License.
-############
-
 import os
 import json
 
 import click
 from cloudify_rest_client.constants import VISIBILITY_EXCEPT_PRIVATE
 
-from .. import env
-from .. import utils
-from ..cli import cfy
-from ..exceptions import CloudifyCliError
-from ..table import print_data, print_details
-from ..utils import (load_json,
-                     print_dict,
-                     validate_visibility,
-                     assert_one_argument,
-                     handle_client_error,
-                     prettify_client_error)
+from cloudify_cli import env, utils
+from cloudify_cli.cli import cfy
+from cloudify_cli.exceptions import CloudifyCliError
+from cloudify_cli.table import print_data, print_details
+from cloudify_cli.utils import (
+    load_json,
+    print_dict,
+    validate_visibility,
+    assert_one_argument,
+    handle_client_error,
+    prettify_client_error)
 
-SECRETS_COLUMNS = ['key', 'created_at', 'updated_at', 'visibility',
-                   'tenant_name', 'created_by', 'is_hidden_value']
+SECRETS_COLUMNS = [
+    'key',
+    'created_at',
+    'updated_at',
+    'visibility',
+    'tenant_name',
+    'created_by',
+    'is_hidden_value',
+    'provider_name',
+]
+
+SECRETS_PROVIDER_COLUMNS = [
+    'name',
+    'type',
+    'visibility',
+    'tenant_name',
+    'created_by',
+    'created_at',
+]
 
 
 @cfy.group(name='secrets')
@@ -45,6 +46,14 @@ def secrets():
         env.raise_uninitialized()
 
 
+@secrets.group(name='providers')
+@cfy.options.common_options
+def providers():
+    """Handle Cloudify Secrets Providers
+    """
+    pass
+
+
 @secrets.command(name='create', short_help='Create a new secret '
                                            '(key-value pair)')
 @cfy.argument('key', callback=cfy.validate_name)
@@ -53,7 +62,14 @@ def secrets():
 @cfy.options.secret_update_if_exists
 @cfy.options.visibility(mutually_exclusive_required=False)
 @cfy.options.hidden_value
+@cfy.options.secret_schema
+@cfy.options.secret_flag_dict
+@cfy.options.secret_flag_list
 @cfy.options.tenant_name(required=False, resource_name_for_help='secret')
+@cfy.options.provider_name
+@cfy.options.provider_options(
+    required=False,
+)
 @cfy.options.common_options
 @cfy.assert_manager_active()
 @cfy.pass_client(use_tenant_in_header=True)
@@ -63,8 +79,13 @@ def create(key,
            secret_file,
            update_if_exists,
            hidden_value,
+           secret_schema,
+           secret_flag_dict,
+           secret_flag_list,
            visibility,
            tenant_name,
+           provider_name,
+           provider_options,
            logger,
            client):
     """Create a new secret (key-value pair)
@@ -73,16 +94,46 @@ def create(key,
     """
     utils.explicit_tenant_name_message(tenant_name, logger)
     validate_visibility(visibility)
-    secret_string = _get_secret_string(secret_file, secret_string)
-    if not secret_string:
+    value = _get_secret_string(secret_file, secret_string)
+    if not value and not provider_name:
         raise CloudifyCliError('Failed to create secret key. '
                                'Missing option '
-                               '--secret-string or secret-file.')
-    client.secrets.create(key,
-                          secret_string,
-                          update_if_exists,
-                          hidden_value,
-                          visibility)
+                               '--secret-string, secret-file or provider.')
+
+    if secret_schema:
+        try:
+            secret_schema = json.loads(secret_schema)
+        except json.decoder.JSONDecodeError as e:
+            raise CloudifyCliError(
+                f'Error decoding JSON schema "{secret_schema}": {e}')
+        if not isinstance(secret_schema, dict) or \
+                not secret_schema.get('type'):
+            raise CloudifyCliError(
+                'Invalid JSON schema. Expected a dict with a "type" key')
+
+    if secret_flag_dict:
+        secret_schema = {"type": "object"}
+    if secret_flag_list:
+        secret_schema = {"type": "array"}
+
+    if secret_schema and value:
+        try:
+            value = json.loads(value)
+        except json.decoder.JSONDecodeError:
+            raise CloudifyCliError(
+                f'Error decoding secret value: \'{value}\' is not of '
+                f'type \'{secret_schema.get("type")}\'')
+
+    client.secrets.create(
+        key,
+        value,
+        update_if_exists,
+        hidden_value,
+        visibility,
+        secret_schema,
+        provider_name,
+        provider_options,
+    )
 
     logger.info('Secret `{0}` created'.format(key))
 
@@ -194,6 +245,10 @@ def import_secrets(passphrase,
 @cfy.options.update_hidden_value
 @cfy.options.update_visibility
 @cfy.options.tenant_name(required=False, resource_name_for_help='secret')
+@cfy.options.provider_name
+@cfy.options.provider_options(
+    required=False,
+)
 @cfy.options.common_options
 @cfy.assert_manager_active()
 @cfy.pass_client(use_tenant_in_header=True)
@@ -204,6 +259,8 @@ def update(key,
            hidden_value,
            visibility,
            tenant_name,
+           provider_name,
+           provider_options,
            logger,
            client):
     """Update an existing secret
@@ -212,10 +269,30 @@ def update(key,
     """
     utils.explicit_tenant_name_message(tenant_name, logger)
     validate_visibility(visibility)
-    secret_string = _get_secret_string(secret_file, secret_string)
+
+    if provider_name:
+        client.secrets_providers.get(provider_name)
+
+    value = _get_secret_string(secret_file, secret_string)
     graceful_msg = 'Requested secret with key `{0}` was not found'.format(key)
     with handle_client_error(404, graceful_msg, logger):
-        client.secrets.update(key, secret_string, visibility, hidden_value)
+        secret_details = client.secrets.get(key)
+        if secret_details.schema:
+            try:
+                value = json.loads(value)
+            except json.decoder.JSONDecodeError:
+                raise CloudifyCliError(
+                    f'Error decoding secret value: \'{value}\' is not of '
+                    f'type \'{secret_details.schema.get("type")}\'')
+
+        client.secrets.update(
+            key,
+            value,
+            visibility,
+            hidden_value,
+            provider_name,
+            provider_options,
+        )
         logger.info('Secret `{0}` updated'.format(key))
 
 
@@ -229,26 +306,43 @@ def update(key,
 @cfy.options.search
 @cfy.options.pagination_offset
 @cfy.options.pagination_size
+@cfy.options.provider_multiple()
 @cfy.assert_manager_active()
 @cfy.pass_client()
 @cfy.pass_logger
 @cfy.options.extended_view
-def list(sort_by,
-         descending,
-         tenant_name,
-         all_tenants,
-         search,
-         pagination_offset,
-         pagination_size,
-         logger,
-         client):
+def _list(
+        sort_by,
+        descending,
+        tenant_name,
+        all_tenants,
+        search,
+        pagination_offset,
+        pagination_size,
+        provider,
+        logger,
+        client,
+):
     """List all secrets
     """
+    filter_rules = None
+
+    if provider:
+        filter_rules = [
+            {
+                "key": "provider_name",
+                "values": provider,
+                "operator": "starts_with",
+                "type": "attribute",
+            }
+        ]
+
     utils.explicit_tenant_name_message(tenant_name, logger)
     logger.info('Listing all secrets...')
     secrets_list = client.secrets.list(
         sort=sort_by,
         is_descending=descending,
+        filter_rules=filter_rules,
         _all_tenants=all_tenants,
         _search=search,
         _offset=pagination_offset,
@@ -295,8 +389,8 @@ def set_global(key, logger, client):
     with prettify_client_error(status_codes, logger):
         client.secrets.set_global(key)
         logger.info('Secret `{0}` was set to global'.format(key))
-        logger.info("This command will be deprecated soon, please use the "
-                    "'set-visibility' command instead")
+        logger.warning("This command is deprecated and will be removed soon, "
+                       "please use the 'set-visibility' command instead")
 
 
 @secrets.command(name='set-visibility',
@@ -339,6 +433,243 @@ def set_owner(key, username, tenant_name, logger, client):
                 key, secret.get('created_by'))
 
 
+@providers.command(
+    name='create',
+    short_help='Create a new Secrets Provider',
+)
+@cfy.argument('secrets_provider_name')
+@cfy.options.secrets_provider_type()
+@cfy.options.secrets_provider_skip_check()
+@cfy.options.connection_parameters(
+    required=False,
+)
+@cfy.options.tenant_name(
+    required=False,
+    resource_name_for_help='Secrets Provider',
+)
+@cfy.options.visibility(
+    mutually_exclusive_required=False,
+)
+@cfy.options.common_options
+@cfy.assert_manager_active()
+@cfy.pass_client(
+    use_tenant_in_header=True,
+)
+@cfy.pass_logger
+def providers_create(
+        secrets_provider_name,
+        secrets_provider_type,
+        skip_check,
+        connection_parameters,
+        tenant_name,
+        visibility,
+        logger,
+        client,
+):
+    client.secrets_providers.create(
+        secrets_provider_name,
+        secrets_provider_type,
+        connection_parameters,
+        visibility,
+    )
+
+    logger.info(
+        'Secrets Provider `%s` created',
+        secrets_provider_name,
+    )
+
+    if not skip_check:
+        client.secrets_providers.check(
+            name=secrets_provider_name,
+        )
+
+        logger.info(
+            'Connected to the Secrets Provider successfully',
+        )
+
+
+@providers.command(
+    name='update',
+    short_help='Update an existing Secrets Provider',
+)
+@cfy.argument('secrets_provider_name')
+@cfy.options.secrets_provider_type(
+    required=False,
+)
+@cfy.options.connection_parameters(
+    required=False,
+)
+@cfy.options.tenant_name(
+    required=False,
+    resource_name_for_help='Secrets Provider',
+)
+@cfy.options.visibility(
+    mutually_exclusive_required=False,
+)
+@cfy.options.common_options
+@cfy.assert_manager_active()
+@cfy.pass_client(
+    use_tenant_in_header=True,
+)
+@cfy.pass_logger
+def providers_update(
+        secrets_provider_name,
+        secrets_provider_type,
+        connection_parameters,
+        tenant_name,
+        visibility,
+        logger,
+        client,
+):
+    client.secrets_providers.update(
+        secrets_provider_name,
+        secrets_provider_type,
+        connection_parameters,
+        visibility,
+    )
+
+    logger.info(
+        'Secrets Provider `%s` updated',
+        secrets_provider_name,
+    )
+
+
+@providers.command(
+    name='delete',
+    short_help='Delete a Secrets Provider',
+)
+@cfy.argument('secrets_provider_name')
+@cfy.options.tenant_name(
+    required=False,
+    resource_name_for_help='Secrets Provider',
+)
+@cfy.options.common_options
+@cfy.assert_manager_active()
+@cfy.pass_client()
+@cfy.pass_logger
+def providers_delete(secrets_provider_name, tenant_name, logger, client):
+    """Delete a Secrets Provider
+    """
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    graceful_msg = 'Requested Secrets Provider with name `{0}` was not found' \
+        .format(secrets_provider_name)
+
+    with handle_client_error(404, graceful_msg, logger):
+        logger.info(
+            'Deleting Secrets Provider `%s`...',
+            secrets_provider_name
+        )
+        client.secrets_providers.delete(secrets_provider_name)
+        logger.info('Secrets Provider removed')
+
+
+@providers.command(
+    name='get',
+    short_help='Get details for a single Secrets Provider',
+)
+@cfy.argument('secrets_provider_name')
+@cfy.options.tenant_name(
+    required=False,
+    resource_name_for_help='Secrets Provider',
+)
+@cfy.options.common_options
+@cfy.assert_manager_active()
+@cfy.pass_client(
+    use_tenant_in_header=True,
+)
+@cfy.pass_logger
+def providers_get(secrets_provider_name, tenant_name, logger, client):
+    """Get details for a single Secrets Provider
+    """
+    utils.explicit_tenant_name_message(tenant_name, logger)
+    graceful_msg = 'Requested Secrets Provider with name `{0}`' \
+                   ' was not found in this tenant'.format(
+                        secrets_provider_name
+                    )
+    with handle_client_error(404, graceful_msg, logger):
+        logger.info(
+            'Getting info for Secrets Provider `%s`...',
+            secrets_provider_name,
+        )
+        details = client.secrets_providers.get(secrets_provider_name)
+
+        print_details(details, 'Requested Secrets Provider info:')
+
+
+@providers.command(
+    name='list',
+    short_help="List all Secrets Providers",
+)
+@cfy.assert_manager_active()
+@cfy.pass_client()
+@cfy.pass_logger
+@cfy.options.extended_view
+@cfy.options.common_options
+def providers_list(
+        logger,
+        client,
+):
+    logger.info('Listing all Secrets Providers...')
+    secrets_list = client.secrets_providers.list()
+    print_data(SECRETS_PROVIDER_COLUMNS, secrets_list, 'Secrets Providers:')
+    total = secrets_list.metadata.pagination.total
+    logger.info(
+        'Showing %s of %s Secrets Providers',
+        len(secrets_list),
+        total,
+    )
+
+
+@providers.command(
+    name='test',
+    short_help='Test a Secrets Provider connectivity',
+)
+@cfy.argument(
+    'secrets_provider_name',
+    required=False,
+    default='',
+)
+@cfy.options.secrets_provider_type(
+    required=False,
+    default='',
+    callback=None,
+)
+@cfy.options.connection_parameters(
+    required=False,
+)
+@cfy.options.tenant_name(
+    required=False,
+    resource_name_for_help='Secrets Provider',
+)
+@cfy.options.visibility(
+    mutually_exclusive_required=False,
+)
+@cfy.options.common_options
+@cfy.assert_manager_active()
+@cfy.pass_client(
+    use_tenant_in_header=True,
+)
+@cfy.pass_logger
+def providers_test(
+        secrets_provider_name,
+        secrets_provider_type,
+        connection_parameters,
+        tenant_name,
+        visibility,
+        logger,
+        client,
+):
+    client.secrets_providers.check(
+        name=secrets_provider_name,
+        _type=secrets_provider_type,
+        connection_parameters=connection_parameters,
+    )
+
+    logger.info(
+        'Connected to the Secrets Provider successfully',
+    )
+
+
 def _get_secret_string(secret_file, secret_string):
     if secret_file:
         if not os.path.exists(secret_file):
