@@ -1,8 +1,11 @@
 import json
 
-from .table import print_data
-from .utils import explicit_tenant_name_message
-from .logger import CloudifyJSONEncoder, get_global_json_output, output
+from cloudify_cli.logger import (
+    CloudifyJSONEncoder,
+    get_global_json_output,
+    output)
+from cloudify_cli.table import print_data
+from cloudify_cli.utils import explicit_tenant_name_message
 
 
 def serialize_resource_labels(resource_list):
