@@ -18,13 +18,12 @@ import os
 import json
 from datetime import datetime
 
+from cloudify_cli.logger import (
+    get_global_json_output,
+    get_global_extended_view,
+    CloudifyJSONEncoder,
+    output)
 from cloudify_cli.prettytable import PrettyTable
-from .logger import (get_global_json_output,
-                     get_global_extended_view,
-                     CloudifyJSONEncoder,
-                     output)
-
-from cloudify._compat import text_type
 
 
 def generate(cols, data, defaults=None, labels=None):
@@ -80,7 +79,7 @@ def generate_extended(cols, data, defaults=None, labels=None):
 
 def get_values_per_column(column, row_data, defaults):
     if column in row_data:
-        if row_data[column] and isinstance(row_data[column], text_type):
+        if row_data[column] and isinstance(row_data[column], str):
             row_data[column] = get_timestamp(row_data[column]) \
                 or row_data[column]
         elif row_data[column] and isinstance(row_data[column], list):
